"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var jsdom_1 = require("jsdom");
var browserResolve = require("lasso-resolve-from");
var context_require_1 = require("context-require");
var remapCache = Object.create(null);
var coverage = global.__coverage__ || (global.__coverage__ = {});
// Expose module.
module.exports = exports = createJSDOMContextRequire;
exports.default = createJSDOMContextRequire;
/**
 * Creates a custom Module object which runs all required scripts
 * in a new jsdom instance.
 */
function createJSDOMContextRequire(options) {
    var html = options.html, dir = options.dir, extensions = options.extensions, beforeParse = options.beforeParse, jsdomOptions = __rest(options, ["html", "dir", "extensions", "beforeParse"]);
    var context = new jsdom_1.JSDOM("", __assign({ runScripts: "dangerously" }, jsdomOptions));
    var window = context.window;
    var resolveConfig = {
        remaps: loadRemaps,
        extensions: extensions &&
            []
                .concat(Object.keys(require.extensions))
                .concat(Object.keys(extensions))
                .filter(unique)
    };
    context.require = context_require_1.default({ dir: dir, context: context, resolve: resolve, extensions: extensions });
    // Pass through istanbul coverage.
    window.__coverage__ = coverage;
    // Expose some globals commonly shimmed by bundlers.
    window.global = window;
    window.Buffer = Buffer;
    window.process = __assign(__assign({}, process), { browser: true });
    window.setImmediate =
        window.setImmediate ||
            (function () {
                var msg = "" + Math.random();
                var queue = [];
                var offset = 0;
                window.addEventListener("message", function (ev) {
                    if (ev.data === msg) {
                        var cbs = queue;
                        offset += cbs.length;
                        queue = [];
                        for (var _i = 0, cbs_1 = cbs; _i < cbs_1.length; _i++) {
                            var cb = cbs_1[_i];
                            if (cb) {
                                cb();
                            }
                        }
                    }
                });
                window.clearImmediate = function clearImmediate(id) {
                    var index = id - offset;
                    if (index >= 0) {
                        queue[index] = undefined;
                    }
                };
                return function setImmediate(cb) {
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    var index = queue.push(args.length ? function () { return cb.apply(void 0, args); } : cb) - 1;
                    if (!index) {
                        window.postMessage(msg, "*");
                    }
                    return index + offset;
                };
            })();
    if (beforeParse) {
        beforeParse(window, context);
    }
    window.document.open();
    window.document.write(html || "<!DOCTYPE html><html><head></head><body></body></html>");
    return context;
    /**
     * A function to resolve modules in the browser using the provided config.
     *
     * @param from The file being resolved from.
     * @param request The requested path to resolve.
     */
    function resolve(from, request) {
        var resolved = browserResolve(from, request, resolveConfig);
        if (!resolved) {
            throw new Error("Could not resolve " + JSON.stringify(request) + " from " + JSON.stringify(from));
        }
        return fs.realpathSync(resolved.path);
    }
}
/**
 * Array filter for uniqueness.
 */
function unique(item, i, list) {
    return list.indexOf(item) === i;
}
/**
 * Loads browser.json remaps.
 *
 * @param dir The directory to load remaps from.
 */
function loadRemaps(dir) {
    var file = path.join(dir, "browser.json");
    if (file in remapCache) {
        return remapCache[file];
    }
    var result;
    var remaps = fs.existsSync(file) && require(file).requireRemap;
    if (remaps) {
        result = {};
        for (var _i = 0, remaps_1 = remaps; _i < remaps_1.length; _i++) {
            var remap = remaps_1[_i];
            result[path.join(dir, remap.from)] = path.join(dir, remap.to);
        }
    }
    remapCache[file] = result;
    return result;
}
//# sourceMappingURL=index.js.map